using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Vevo.Shared.DataAccess;

namespace Vevo.DataAccessLib.Cart
{
    public static class AdminMenuAdvancedPermissionAccess
    {
        private const string AllMenuFields = "AdminMenuAdvancedPermission.MenuPageName, SortOrder, IsEnabled, " +
    "MenuName ";

        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "menupagename" ))
                return "ORDER BY AdminMenuAdvancedPermission." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        public static DataTable GetAll( string cultureID, string sortBy, FlagFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllMenuFields + " FROM " +
                "AdminMenuAdvancedPermission LEFT JOIN AdminMenuAdvancedPermissionLocale ON " +
                "AdminMenuAdvancedPermission.MenuPageName = AdminMenuAdvancedPermissionLocale.MenuPageName " +
                "WHERE CultureID = @CultureID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", isEnabled ) +
                CreateSortByText( sortBy ),
                DataAccess.CreateParameterString( cultureID ) );
            return table;
        }

        public static DataTable GetAllByAdminID( string cultureID, string adminID, string sortBy, FlagFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllMenuFields + ", ViewMode, ModifyMode FROM " +
                "( AdminMenuAdvancedPermission LEFT JOIN AdminMenuAdvancedPermissionLocale ON " +
                "AdminMenuAdvancedPermission.MenuPageName = AdminMenuAdvancedPermissionLocale.MenuPageName ) " +
                "LEFT JOIN AdminMenuPermission ON " +
                "AdminMenuAdvancedPermission.MenuPageName = AdminMenuPermission.MenuPageName " +
                "WHERE CultureID = @CultureID AND AdminID = @AdminID AND ViewMode = @ViewMode " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", isEnabled ) +
                CreateSortByText( sortBy ),
                DataAccess.CreateParameterString( cultureID ),
                DataAccess.CreateParameterString( adminID ),
                DataAccess.CreateParameterBool( true ) );
            return table;
        }
    }
}
