﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Returns;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class RmaRepositoryNoCache : IRmaRepository
    {
        #region Private

        private const string _allField = "RmaID, StoreID, CustomerID, RequestDate, OrderID, ProductName, Quantity, " +
            "ReturnReason, RmaActionID, RequestStatus, RmaNote ";
        private const string _allRmaCustomerField = "RmaID, StoreID, Rma.CustomerID, RequestDate, OrderID, ProductName, Quantity, " +
            "ReturnReason, RmaActionID, RequestStatus, RmaNote, UserName ";
        private const string _rmaHistoryField = "RmaID, OrderID, ProductName, RequestStatus ";

        private Rma Create( Rma rma )
        {
            string rmaID = DataAccess.ExecuteScalar( "INSERT INTO Rma( " +
                "StoreID, CustomerID, RequestDate, OrderID, ProductName, Quantity, " +
                "ReturnReason, RmaActionID, RequestStatus, RmaNote ) VALUES ( " +
                "@StoreID, @CustomerID, @RequestDate, @OrderID, @ProductName, @Quantity, " +
                "@ReturnReason, @RmaActionID, @RequestStatus, @RmaNote ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( rma.StoreID ),
                DataAccess.CreateParameterString( rma.CustomerID ),
                DataAccess.CreateParameterDateTime( rma.RequestDate ),
                DataAccess.CreateParameterString( rma.OrderID ),
                DataAccess.CreateParameterString( rma.ProductName ),
                DataAccess.CreateParameterInt32( rma.Quantity ),
                DataAccess.CreateParameterString( rma.ReturnReason ),
                DataAccess.CreateParameterString( rma.RmaActionID ),
                DataAccess.CreateParameterString( rma.RequestStatus ),
                DataAccess.CreateParameterString( rma.RmaNote )
            );

            rma.RmaID = rmaID;

            return rma;
        }

        private Rma Update( Rma rma )
        {
            string sql = "UPDATE Rma SET StoreID = @StoreID, CustomerID = @CustomerID, RequestDate = @RequestDate, " +
                "OrderID = @OrderID, ProductName = @ProductName, Quantity = @Quantity, ReturnReason = @ReturnReason," +
                "RmaActionID = @RmaActionID, RequestStatus = @RequestStatus, RmaNote = @RmaNote WHERE RmaID = @RmaID; ";

            DataAccess.ExecuteNonQuery( sql,
                DataAccess.CreateParameterString( rma.StoreID ),
                DataAccess.CreateParameterString( rma.CustomerID ),
                DataAccess.CreateParameterDateTime( rma.RequestDate ),
                DataAccess.CreateParameterString( rma.OrderID ),
                DataAccess.CreateParameterString( rma.ProductName ),
                DataAccess.CreateParameterInt32( rma.Quantity ),
                DataAccess.CreateParameterString( rma.ReturnReason ),
                DataAccess.CreateParameterString( rma.RmaActionID ),
                DataAccess.CreateParameterString( rma.RequestStatus ),
                DataAccess.CreateParameterString( rma.RmaNote ),
                DataAccess.CreateParameterString( rma.RmaID ) );

            return rma;
        }

        private string CreateSortByText( string sortBy )
        {
            if ( String.IsNullOrEmpty( sortBy ) )
                return String.Empty;
            else
                return "ORDER BY " + sortBy;
        }

        private Rma CreateOneFromRow( DataRow row )
        {
            Rma rma = new Rma();
            rma.RmaID = row[ "RmaID" ].ToString();
            rma.StoreID = row[ "StoreID" ].ToString();
            rma.CustomerID = row[ "CustomerID" ].ToString();
            rma.RequestDate = ConvertUtilities.ToDateTime( row[ "RequestDate" ] );
            rma.OrderID = row[ "OrderID" ].ToString();
            rma.ProductName = row[ "ProductName" ].ToString();
            rma.Quantity = ConvertUtilities.ToInt32( row[ "Quantity" ] );
            rma.ReturnReason = row[ "ReturnReason" ].ToString();
            rma.RmaActionID = row[ "RmaActionID" ].ToString();
            rma.RequestStatus = row[ "RequestStatus" ].ToString();
            rma.RmaNote = row[ "RmaNote" ].ToString();            
            rma.UserName = rma.GetCustomer.UserName;

            return rma;
        }

        private string CreateStoreFilter( string storeID )
        {
            if ( !String.IsNullOrEmpty( storeID ) )
                return "AND StoreID = " + storeID + " ";
            else
                return " ";
        }

        private string CreateStatusFilter( Rma.RmaStatus status )
        {
            if ( status != Rma.RmaStatus.All )
                return "AND RequestStatus = '" + status.ToString() + "' ";
            else
                return " ";
        }

        private IList<Rma> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<Rma> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<Rma> list = new List<Rma>();

            for ( int i = startIndex; i <= endIndex && i < table.Rows.Count; i++ )
            {
                list.Add( CreateOneFromRow( table.Rows[ i ] ) );
            }

            return list;
        }

        private IList<Rma> SearchRmaPrivate(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _allField +
                "FROM Rma " + commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns[ "RmaID" ] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        private IList<Rma> SearchRmaPrivate(
            string sortBy,
            SearchFilter searchFilter,
            string storeID,
            string customerID,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithAnd(
                searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _allField +
                "FROM Rma " +
                "WHERE CustomerID = " + customerID + " " +
                commandSearch +
                CreateStoreFilter( storeID ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns[ "RmaID" ] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table );
        }

        private IList<Rma> SearchRmaPrivate(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            string storeID,
            Rma.RmaStatus status,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithAnd(
                searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _allRmaCustomerField +
                "FROM Rma, Customer " +
                "WHERE Rma.CustomerID = Customer.CustomerID " +
                commandSearch +
                CreateStoreFilter( storeID ) +
                CreateStatusFilter( status ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns[ "RmaID" ] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        private IList<Rma> SearchRmaPrivate(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            string storeID,
            string customerID,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithAnd(
                searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _allField +
                "FROM Rma " +
                "WHERE CustomerID = " + customerID + " " +
                commandSearch +
                CreateStoreFilter( storeID ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns[ "RmaID" ] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        #endregion

        #region Public Methods

        public Rma Save( Rma rma )
        {
            Rma result;

            if ( rma.RmaID == "0" )
                result = Create( rma );
            else
                result = Update( rma );

            return result;
        }

        public Rma GetOne( string rmaID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _allField +
                "FROM Rma " +
                "WHERE RmaID = @RmaID; ",
                DataAccess.CreateParameterString( rmaID ) );

            if ( table.Rows.Count > 0 )
                return CreateOneFromRow( table.Rows[ 0 ] );
            else
                return Rma.Null;
        }

        public void Delete( string rmaID )
        {
            using ( DataTransactionScope transaction = new DataTransactionScope() )
            {
                DataAccess.ExecuteNonQuery( "DELETE FROM Rma WHERE RmaID = @RmaID; ",
                DataAccess.CreateParameterString( rmaID ) );

                transaction.Complete();
            }
        }

        public IList<Rma> SearchRma(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            return SearchRmaPrivate( sortBy, searchFilter, startIndex, endIndex, out howManyItems );
        }

        public IList<Rma> SearchRma(
            string sortBy,
            SearchFilter searchFilter,
            string storeID,
            string customerID,
            out int howManyItems )
        {
            return SearchRmaPrivate( sortBy, searchFilter, storeID, customerID, out howManyItems );
        }

        public IList<Rma> SearchRma(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            string storeID,
            Rma.RmaStatus status,
            out int howManyItems )
        {
            return SearchRmaPrivate( sortBy, searchFilter, startIndex, endIndex, storeID, status, out howManyItems );
        }

        public IList<Rma> SearchRma(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            string storeID,
            string customerID,
            out int howManyItems )
        {
            return SearchRmaPrivate( sortBy, searchFilter, startIndex, endIndex, storeID, customerID, out howManyItems );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT " + _allField +
                 "FROM Rma " +
                 "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<TableSchemaItem> GetRmaHistoryTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT " + _rmaHistoryField +
                 "FROM Rma " +
                 "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
