using System;
using System.Collections;
using System.Text;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.ImportExport
{
    /// <summary>
    /// Summary description for ProductImportStatus
    /// </summary>
    public class ProductImportStatus
    {
        public enum ErrorStatus { Success, DoneWithErrors, Fatal }

        public ErrorStatus Status
        {
            get
            {
                return _status;
            }
        }

        public string ProductNameError;
        public ArrayList ProductNameErrorList;
        public string RelatedProductError;
        public int ImportedCount;
        public int ErrorCount;
        public ArrayList RelatedErrorList;
        public string CategoryError;
        public ArrayList CategoryErrorList;
        public string DepartmentError;
        public ArrayList DepartmentErrorList;
        public ArrayList ImageUploadErrorList;
        public string ImageUploadError;
        public ArrayList OtherErrorList;
        public ArrayList OtherErrorExceptionList;
        public string OtherError;
        public string UseOptionError;
        public ArrayList UseOptionErrorList;
        public string ImageProcessError;
        public ArrayList ImageProcessErrorList;
        public ArrayList RecurringGiftDownloadErrorList;
        public string RecurringGiftDownloadError;
        public ArrayList RecurringIntervalCyclesErrorList;
        public string RecurringIntervalCyclesError;
        public ArrayList RecurringIntervalUnitErrorList;
        public string RecurringIntervalUnitError;
        public ArrayList TaxClassErrorList;
        public string TaxClassError;
        public ArrayList SKUImportedErrorList;
        public string SKUImportedError;
        public ArrayList MinMaxQuantityErrorList;
        public string MinMaxQuantityError;
        public ArrayList SpecificationItemErrorList;
        public string SpecificationItemError;
        public ArrayList ProductSKUErrorList;
        public string ProductSKUError;
        public string ProductKitGroupError;
        public ArrayList ProductKitGroupErrorList;
        public string InventoryAndStockError;
        public ArrayList InventoryAndStockErrorList;
        public string ProductKitQuantityError;
        public ArrayList ProductKitQuantityErrorList;

        public string ErrorMessage
        {
            get
            {
                if ( ErrorCount > 0 || _status == ErrorStatus.Fatal )
                    return _errorBuilder.ToString();
                else
                    return String.Empty;
            }
        }

        public ProductImportStatus()
        {
            ImportedCount = 0;
            ErrorCount = 0;
            ProductNameError = String.Empty;
            ProductNameErrorList = new ArrayList();
            RelatedProductError = String.Empty;
            RelatedErrorList = new ArrayList();
            CategoryError = String.Empty;
            CategoryErrorList = new ArrayList();
            DepartmentError = String.Empty;
            DepartmentErrorList = new ArrayList();
            ImageUploadErrorList = new ArrayList();
            OtherErrorList = new ArrayList();
            OtherErrorExceptionList = new ArrayList();
            UseOptionError = String.Empty;
            UseOptionErrorList = new ArrayList();
            ImageProcessError = String.Empty;
            ImageProcessErrorList = new ArrayList();
            RecurringGiftDownloadError = String.Empty;
            RecurringGiftDownloadErrorList = new ArrayList();
            RecurringIntervalCyclesError = String.Empty;
            RecurringIntervalCyclesErrorList = new ArrayList();
            RecurringIntervalUnitError = String.Empty;
            RecurringIntervalUnitErrorList = new ArrayList();
            TaxClassErrorList = new ArrayList();
            TaxClassError = String.Empty;
            SKUImportedErrorList = new ArrayList();
            SKUImportedError = String.Empty;
            MinMaxQuantityErrorList = new ArrayList();
            MinMaxQuantityError = String.Empty;
            SpecificationItemErrorList = new ArrayList();
            SpecificationItemError = String.Empty;
            ProductSKUErrorList = new ArrayList();
            ProductSKUError = String.Empty;
            ProductKitGroupError = String.Empty;
            ProductKitGroupErrorList = new ArrayList();
            InventoryAndStockError = String.Empty;
            InventoryAndStockErrorList = new ArrayList();
            ProductKitQuantityError = String.Empty;
            ProductKitQuantityErrorList = new ArrayList();
        }

        private StringBuilder _errorBuilder = new StringBuilder( "Error(s) sku in the following row(s) of the CSV file:\n" );
        private ErrorStatus _status = ErrorStatus.Success;

        public void AddProductNameError( string lineNumber )
        {
            ProductNameErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddSKUImportedError( string lineNumber )
        {
            SKUImportedErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddImageUploadError( string lineNumber )
        {
            ImageUploadErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddRelatedProductError( string lineNumber )
        {
            RelatedErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddCategoryError( string lineNumber )
        {
            CategoryErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddDepartmentError( string lineNumber )
        {
            DepartmentErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddUseOptionError( string lineNumber )
        {
            UseOptionErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddImageProcessError( string fileName )
        {
            ImageProcessErrorList.Add( fileName );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddRecurringGiftDownloadError( string lineNumber )
        {
            RecurringGiftDownloadErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            AddError();
        }

        public void AddRecurringIntervalCyclesError( string lineNumber )
        {
            RecurringIntervalCyclesErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            AddError();
        }

        public void AddRecurringIntervalUnitError( string lineNumber )
        {
            RecurringIntervalUnitErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            AddError();
        }

        public void AddTaxClassError( string lineNumber )
        {
            TaxClassErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddMinMaxQuantityError( string lineNumber )
        {
            MinMaxQuantityErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddSpecificationItemError( string lineNumber )
        {
            SpecificationItemErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddProductSKUError( string lineNumber )
        {
            ProductSKUErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddProductKitGroupError( string lineNumber )
        {
            ProductKitGroupErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddInventoryAndStockError( string lineNumber )
        {
            InventoryAndStockErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddProductKitQuantityError( string lineNumber )
        {
            ProductKitQuantityErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddSuccess()
        {
            ImportedCount++;
        }

        public void AddError()
        {
            ErrorCount++;
        }

        public void AddOtherError( string lineNumber, Exception ex )
        {
            OtherErrorList.Add( lineNumber );
            OtherErrorExceptionList.Add( ex );

            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void SetFatalError( string message )
        {
            _errorBuilder = new StringBuilder( message );
            _status = ErrorStatus.Fatal;
        }

        private string CreateOtherErrorDetails()
        {
            StringBuilder sb = new StringBuilder();
            for ( int i = 0; i < 5 && i < OtherErrorExceptionList.Count; i++ )
            {
                if ( i > 0 )
                    sb.Append( "<br/>----------<br/>" );

                Exception ex = ( Exception ) OtherErrorExceptionList[ i ];

                sb.Append( "<br/>Message: " );
                sb.Append( ex.Message );
                sb.Append( "<br/>Exception Type: " );
                sb.Append( ex.GetType().ToString() );
                sb.Append( "<br/>Stack Trace:<br/>" );
                sb.Append( HtmlFormatter.ConvertNewLines( ex.StackTrace ) );
            }

            return sb.ToString();
        }

        public void BuildOtherError()
        {
            if ( OtherErrorList.Count != 0 )
            {
                string[] result = new string[ OtherErrorList.Count ];
                OtherErrorList.CopyTo( result );
                OtherError = String.Format(
                    "Other error(s) in the following line number(s) in your csv file:<br />{0}<br/><br/>" +
                    "Details of the first few error(s):<br/>{1}",
                    String.Join( ", ", result ),
                    CreateOtherErrorDetails() );
            }
        }

        public void BuildProductNameError()
        {
            if ( ProductNameErrorList.Count != 0 )
            {
                string[] result = new string[ ProductNameErrorList.Count ];
                ProductNameErrorList.CopyTo( result );
                ProductNameError =
                    "Error(s) in product name field:<br/>" +
                    "Invalid Product Name for the following line number(s) in your csv file:<br/>" +
                    String.Join( ", ", result );
            }
        }

        public void BuildRelatedProductError()
        {
            if ( RelatedErrorList.Count != 0 )
            {
                string[] result = new string[ RelatedErrorList.Count ];
                RelatedErrorList.CopyTo( result );
                RelatedProductError =
                    "Error(s) in related products field:<br/>" +
                    "Invalid RelatedProducts for the following line number(s) in your csv file:<br/>" +
                    String.Join( ", ", result );
            }
        }

        public void BuildSKUImportedError()
        {
            if ( SKUImportedErrorList.Count != 0 )
            {
                string[] result = new string[ SKUImportedErrorList.Count ];
                SKUImportedErrorList.CopyTo( result );
                SKUImportedError = "Error(s) in SKU field:<br />" +
                    "Same SKU Product imported for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildCategoryError()
        {
            if ( CategoryErrorList.Count != 0 )
            {
                string[] result = new string[ CategoryErrorList.Count ];
                CategoryErrorList.CopyTo( result );
                CategoryError = "Error(s) in category field:<br />" +
                    "Invalid category not exists for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildDepartmentError()
        {
            if ( DepartmentErrorList.Count != 0 )
            {
                string[] result = new string[ DepartmentErrorList.Count ];
                DepartmentErrorList.CopyTo( result );
                DepartmentError = "Error(s) in department field:<br />" +
                    "Invalid department not exists for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildImageUploadError()
        {
            if ( ImageUploadErrorList.Count != 0 )
            {
                string[] result = new string[ ImageUploadErrorList.Count ];
                ImageUploadErrorList.CopyTo( result );
                CategoryError = "Error(s) in image field:<br />" +
                    "Images files specified in csv do not exist for the following line number(s)<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildUseOptionError()
        {
            if ( UseOptionErrorList.Count != 0 )
            {
                string[] result = new string[ UseOptionErrorList.Count ];
                UseOptionErrorList.CopyTo( result );
                UseOptionError = "Error(s) in UseOption, OptionGroupIDx, UseStockOptionx, OptionItemIDx field:<br/>" +
                    "Please verify values of one or more fields in your csv file:<br/>" +
                    "Invalid use option setting for the following line number(s).<br/>" + String.Join( ", ", result );
            }
        }

        public void BuildImageProcessError()
        {
            if ( ImageProcessErrorList.Count != 0 )
            {
                string[] result = new string[ ImageProcessErrorList.Count ];
                ImageProcessErrorList.CopyTo( result );
                ImageProcessError = "Error reading the following image file(s):<br/>" +
                    String.Join( ", ", result );
            }
        }

        public void BuildRecurringGiftDownloadError()
        {
            if ( RecurringGiftDownloadErrorList.Count != 0 )
            {
                string[] result = new string[ RecurringGiftDownloadErrorList.Count ];
                RecurringGiftDownloadErrorList.CopyTo( result );
                RecurringGiftDownloadError =
                    "Error(s) in IsRecurring, IsDownloadable, IsGiftCertificate fields:<br/>" +
                    "Recurring products cannot be downloadable products or gift certificates.<br/>" +
                    "Please verify values of the following line number(s) in your csv file.<br/>" + String.Join( ", ", result );
            }
        }

        public void BuildRecurringIntervalCyclesError()
        {
            if ( RecurringIntervalCyclesErrorList.Count > 0 )
            {
                string[] result = new string[ RecurringIntervalCyclesErrorList.Count ];
                RecurringIntervalCyclesErrorList.CopyTo( result );
                RecurringIntervalCyclesError =
                    "Error(s) in RecurringInterval, RecurringNumberOfCycles fields:<br/>" +
                    "Recurring interval and number of cycles must be greater than zero.<br/>" +
                    "Please verify values of the following line number(s) in your csv file.<br/>" + String.Join( ", ", result );
            }
        }

        public void BuildRecurringIntervalUnitError()
        {
            if ( RecurringIntervalUnitErrorList.Count > 0 )
            {
                string[] result = new string[ RecurringIntervalUnitErrorList.Count ];
                RecurringIntervalUnitErrorList.CopyTo( result );
                RecurringIntervalUnitError =
                    "Error(s) in RecurringIntervalUnit fields:<br/>" +
                    "IntervalUnit must be 'Day', 'Week', 'Month', or 'Year'.<br/>" +
                    "Please verify values of the following line number(s) in your csv file.<br/>" + String.Join( ", ", result );
            }
        }

        public void BuildTaxClassError()
        {
            if ( TaxClassErrorList.Count != 0 )
            {
                string[] result = new string[ TaxClassErrorList.Count ];
                TaxClassErrorList.CopyTo( result );
                TaxClassError = "Error(s) in TaxClass field:<br />" +
                    "Invalid TaxClass not exists for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildMinMaxQuantityError()
        {
            if ( MinMaxQuantityErrorList.Count > 0 )
            {
                string[] result = new string[ MinMaxQuantityErrorList.Count ];
                MinMaxQuantityErrorList.CopyTo( result );
                MinMaxQuantityError =
                    "Error(s) in Minimum Quantity and Maximum Quantity fields:<br/>" +
                    "Minimum and Maximum quantity must grater than zero except maximum quantity can set to zero for unlimited.<br/>" +
                    "Please verify values of the following line number(s) in your csv file.<br/>" + String.Join( ", ", result );
            }
        }

        public void BuildSpecificationItemError()
        {
            if ( SpecificationItemErrorList.Count != 0 )
            {
                string[] result = new string[ SpecificationItemErrorList.Count ];
                SpecificationItemErrorList.CopyTo( result );
                SpecificationItemError = "Error(s) in SpecificationItemID field:<br />" +
                    "Invalid Specification Item ID not exists for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildProductSKUError()
        {
            if ( ProductSKUErrorList.Count != 0 )
            {
                string[] result = new string[ ProductSKUErrorList.Count ];
                ProductSKUErrorList.CopyTo( result );
                ProductSKUError = "Error(s) in SKU field:<br />" +
                    "Invalid product SKU not exists for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildProductKitGroupError()
        {
            if ( ProductKitGroupErrorList.Count != 0 )
            {
                string[] result = new string[ ProductKitGroupErrorList.Count ];
                ProductKitGroupErrorList.CopyTo( result );
                ProductKitGroupError = "Error(s) in ProductKitGroup field:<br />" +
                    "Invalid Product Kit Group not exists for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildInventoryAndStockError()
        {
            if ( InventoryAndStockErrorList.Count != 0 )
            {
                string[] result = new string[ InventoryAndStockErrorList.Count ];
                InventoryAndStockErrorList.CopyTo( result );
                InventoryAndStockError = "Error(s) in Product Inventory field:<br />" +
                    "Invalid UseInventory or Stock not exists for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }

        public void BuildProductKitQuantityError()
        {
            if (ProductKitQuantityErrorList.Count != 0)
            {
                string[] result = new string[ProductKitQuantityErrorList.Count];
                ProductKitQuantityErrorList.CopyTo( result );
                ProductKitQuantityError = "Error(s) in Quantity field:<br />" +
                    "Invalid Quantity. This field cannot be character, negative value or emptied for the following line number(s) in your csv file:<br />" +
                    String.Join( ", ", result );
            }
        }
    }
}