﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.Products;
using Vevo.Domain.Users;
using Vevo.Domain.Marketing;
using Vevo.Domain.Stores;

namespace Vevo.Domain.Orders
{
    public interface ICartItem
    {
        #region Public Properties

        //---- Cart Item Properties ---
        bool IsNull { get; }
        string CartItemID { get; }
        string CartID { get; }

        //--- Product Related Properties ---
        Product Product { get; }
        string ProductID { get; }
        bool IsCustomPrice { get; }
        decimal ProductCustomPrice { get; }
        decimal EnteredPrice { get; set; }
        bool IsEnterPrice { get; set; }
        int Quantity { get; set; }
        bool FixedShipping { get; }
        bool IsDownloadable { get; }
        bool IsSubscriptionable { get; set; }
        double UnitWeight { get; }
        string DiscountGroupID { get; }
        string OptionCombinationID { get; }
        bool IsGiftCertificate { get; }
        CartItemGiftDetails GiftDetails { get; }
        decimal GiftPrice { get; }
        OptionItemValueCollection Options { get; }
        bool IsTaxable { get; }
        string TaxClassID { get; }
        bool IsRecurring { get; }
        int RecurringInterval { get; }
        string RecurringIntervalUnit { get; }
        int RecurringNumberOfCycles { get; }
        int RecurringNumberOfTrialCycles { get; }
        bool IsProductKit { get; }
        ProductKitItemValueCollection ProductKits { get; }

        //---- Promotional Related Properties ----
        bool IsPromotion { get; }
        //IPromotionSelected PromotionSelected { get; }
        #endregion


        #region Public Methods
        string GetName( Culture culture, Currency currency);
        string GetNameByStoreID( Culture culture, Currency currency ,string storeID);
        string GetOrderItemName(
            Culture culture,
            Currency currency,
            OrderAmountPackage orderAmountPackage );
        string GetPreCheckoutTooltip(
            Currency currency,
            WholesaleStatus wholesaleStatus,
            out string mainText );
        string GetPreCheckoutTooltip(
           Currency currency,
           WholesaleStatus wholesaleStatus,
           string recurringTooltipMainText,
           string recurringProductDescription,
           string recurringShoppingCartWarning,
           string storeID,
           out string mainText );

        decimal GetShoppingCartUnitPrice( WholesaleStatus wholesaleStatus );
        decimal GetCheckoutUnitPrice( WholesaleStatus wholesaleStatus );
        decimal GetUnitPriceWithoutProductOption( WholesaleStatus wholesaleStatus );
        decimal GetCheckoutIncludedTaxAmount( WholesaleStatus wholesaleStatus );
        decimal GetRecurringRegularUnitPrice( WholesaleStatus wholesaleStatus );
        decimal GetRecurringRegularUnitPrice( WholesaleStatus wholesaleStatus, string storeID );
        decimal GetRecurringTrialUnitPrice( WholesaleStatus wholesaleStatus );
        decimal GetSubtotal( WholesaleStatus wholesaleStatus );
        decimal GetSubtotalIncludedTax( WholesaleStatus wholesaleStatus );
        bool IsShippable();
        bool ValidateReferredObjects();

        #endregion
    }
}
