﻿using System;
using System.Data;
using System.Collections;
using System.Configuration;
using System.Text;
using System.Text.RegularExpressions;

using Vevo.Domain.Exceptions;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Orders
{
    [Serializable]
    public class ProductKitItemValueCollection : CollectionBase
    {
        #region Private

        private const string ItemSeparator = ",";

        private bool CompareCondition( ProductKitItemValue source, ProductKitItemValue compare )
        {
            // For Option Input List, "Details" field will contain quantity (e.g. 1).
            // Therefore, the comparison may not be the same even though OptionItemID and
            // UseStock are the same.
            if ((source.ProductKitItem.ProductKitGroupID == compare.ProductKitItem.ProductKitGroupID) &&
             (source.ProductKitItem.ProductID == compare.ProductKitItem.ProductID) && (source.Quantity == compare.Quantity))
                return true;
            else
                return false;       
        }

        #endregion

        #region Constructors
        public ProductKitItemValueCollection()
        {
        }

        public ProductKitItemValueCollection( ProductKitItemValue[] kitItemValue )
        {
            foreach (ProductKitItemValue item in kitItemValue)
            {
                this.Add( item );
            }
        }
        #endregion

        #region Public Properties

        // Return null object
        public static ProductKitItemValueCollection Null
        {
            get { return NullProductStockItemValueCollection.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        #endregion

        #region Public Methods

        public override string ToString()
        {
            string result = String.Empty;

            foreach (ProductKitItemValue itemValue in this)
            {
                result += itemValue.Details + " x" + itemValue.Quantity.ToString() + ItemSeparator;
            }

            return result;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        public override bool Equals( object obj )
        {
            ProductKitItemValueCollection compareList = (ProductKitItemValueCollection) obj;

            if (this.Count != compareList.Count)
            {
                return false;
            }
            else
            {
                foreach (ProductKitItemValue source in this)
                {
                    bool checkName = false;
                    foreach (ProductKitItemValue compare in compareList)
                    {
                        if (source.ProductKitGroupID == compare.ProductKitGroupID)
                        {
                            checkName = true;
                            break;
                        }
                    }

                    if (checkName == false)
                    {
                        return false;
                    }
                }

                bool equal = true;

                foreach (ProductKitItemValue source in this)
                {
                    bool result = true;
                    foreach (ProductKitItemValue compare in compareList)
                    {
                        if (source.ProductKitGroupID == compare.ProductKitGroupID)
                        {
                            if (!CompareCondition( source, compare ))
                            {
                                result = false;
                                break;
                            }
                        }
                    }

                    if (result == false)
                    {
                        equal = false;
                        break;
                    }
                }

                return equal;
            }
        }

        public virtual void Add( ProductKitItemValue optionItem )
        {
            this.InnerList.Add( optionItem );
        }

        public ProductKitItemValue this[int index]
        {
            get
            {
                return (ProductKitItemValue) this.InnerList[index];
            }
            set
            {
                this.InnerList[index] = value;
            }
        }

        public ProductKitItemValueCollection Clone()
        {
            ProductKitItemValueCollection collection = new ProductKitItemValueCollection();

            foreach (ProductKitItemValue item in this)
            {
                collection.Add( item );
            }

            return collection;
        }

        public string[] ConvertToIDArray()
        {
            ArrayList list = new ArrayList();

            foreach (ProductKitItemValue item in this)
            {
                list.Add( item.Details + " x" + item.Quantity.ToString() );
            }

            string[] result = new string[list.Count];
            list.CopyTo( result );
            return result;
        }

        #endregion
    }
}
