using System;
using System.Collections.Generic;
using Vevo.Domain;
using Vevo.Domain.Users;
using Vevo.QuickBooks.Qbxml70;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for CustomerQueryRqState
    /// </summary>
    public class CustomerQueryRequestState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new CustomerQueryResponseState( QBWCSession );
        }

        private void MoveToCustomerModifyRqState()
        {
            QBWCSession.CurrentState = new CustomerModifyRequestState( QBWCSession );
        }

        private IList<Customer> GetExportData()
        {
            IList<Customer> table = DataAccessContext.CustomerRepository.GetAll( String.Empty );
            return table;
        }

        private string GetCustomerName( Customer customer )
        {
            string name = string.Empty;
            if ( DataAccessContext.Configurations.GetValue( "QBCustomerNameFormat" ).Equals( "lastfirst" ) )
                name = customer.BillingAddress.LastName + ", " + customer.BillingAddress.FirstName;
            else
                name = customer.BillingAddress.FirstName + " " + customer.BillingAddress.LastName;
            
            return QBUtilities.LimitLength( name, QBConst.CustomerNameMaxLength );
        }

        private CustomerQueryRqType CreateOneRequest( Customer customer )
        {
            CustomerQueryRqType req = new CustomerQueryRqType();
            NameRangeFilter filter = new NameRangeFilter();
            string customerName = GetCustomerName( customer );
            filter.FromName = customerName;
            filter.ToName = customerName + "z";
            req.Items = new object[] { filter };
            req.ItemsElementName = new ItemsChoiceType25[] { ItemsChoiceType25.NameRangeFilter };

            return req;
        }

        private CustomerQueryRqType CreateAnonymousRequest()
        {
            CustomerQueryRqType req = new CustomerQueryRqType();
            NameRangeFilter filter = new NameRangeFilter();
            filter.FromName = "Online Anonymous";
            filter.ToName = "Online Anonymousz";
            req.Items = new object[] { filter };
            req.ItemsElementName = new ItemsChoiceType25[] { ItemsChoiceType25.NameRangeFilter };

            return req;
        }

        private string CreateCustomerQuery( out bool xmlReady )
        {
            // Write QBXML
            IList<Customer> table = GetExportData();
            int lenght = table.Count;
            if ( DataAccessContext.Configurations.GetValue( "QBAnonymousListID" ).Equals( "" ) )
                lenght++;

            if (lenght == 0)
            {
                xmlReady = false;
                MoveToCustomerModifyRqState();
                return String.Empty;
            }

            QBXML qbxml = CreateRequestQbxml();

            object[] requests = new object[lenght];
            int index = 0;
            foreach ( Customer customer in table )
            {
                requests[index] = CreateOneRequest( customer );
                index++;
            }

            if (DataAccessContext.Configurations.GetValue( "QBAnonymousListID" ).Equals( "" ) )
                requests[index] = CreateAnonymousRequest();

            ((QBXMLMsgsRq) qbxml.Items[0]).Items = requests;

            xmlReady = true;
            return CreateRequestXml( qbxml );
        }

        public CustomerQueryRequestState( QBWCSession session )
            : base( session )
        {
        }

        public override string OnSendRequestXml( out bool xmlReady )
        {
            MoveToNextState();

            return CreateCustomerQuery( out xmlReady );
        }

    }
}
