using System;
using System.Collections;
using System.Data;
using System.Text;
using Vevo.Shared.DataAccess;
using Vevo.Shared.DataAccess.Implementation;

namespace Vevo.Reports
{
    public class SaleReportBuilder
    {
        private const int DayMonthConversionXAxisThreshold = 60;       // Number in days

        private string CreateTotalSqlNoAs()
        {
            return " (Subtotal + Tax + ShippingCost + HandlingFee - CouponDiscount - GiftCertificate) ";
        }

        private string CreateSortString( string sortBy )
        {
            if (sortBy.Contains( "AvgQuantity" ))
            {
                string[] sort = sortBy.Split( ' ' );
                return "Order BY " +
                    DbImplementation.Sql.CreateDivideFucntion( "Quantity", "NumberOfOrder" ) +
                    sort[1];
            }
            else if (sortBy.Contains( "Quantity" ))
            {
                return "ORDER BY " + DbImplementation.Sql.AddTablePrefixIfNecessary( sortBy, "OI" );
            }
            else
            {
                return "ORDER BY " + DbImplementation.Sql.AddTablePrefixIfNecessary( sortBy, "O" );
            }
        }

        private DataTable GetOneDayTable(
            string sortBy,
            DateTime? startOrderDate,
            DateTime? endOrderDate,
            string reportType,
            string storeID
            )
        {
            string sort, from1, from2, to1, to2;
            ArrayList parameters = new ArrayList();

            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );
            if (reportType == "24Hours")
            {
                from1 = DateFilterHelper.CreateFromDateFor24HoursWithAnd(
                    startOrderDate, parameters, "From1", "OrderDate" );
                to1 = DateFilterHelper.CreateToDateFor24HoursWithAnd(
                    endOrderDate, parameters, "To1", "OrderDate" );
            }
            else
            {
                from1 = DateFilterHelper.CreateFromDateWithAnd(
                    startOrderDate, parameters, "From1", "OrderDate" );
                to1 = DateFilterHelper.CreateToDateWithAnd(
                    endOrderDate, parameters, "To1", "OrderDate" );
            }

            string storeFilter1 = "";
            if (storeID != "0")
            {
                storeFilter1 = "AND StoreID = @StoreID1 ";
                parameters.Add( DataAccess.CreateParameterString( storeID ) );
            }

            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );
            if (reportType == "24Hours")
            {
                from2 = DateFilterHelper.CreateFromDateFor24HoursWithAnd(
                    startOrderDate, parameters, "From2", "OrderDate" );
                to2 = DateFilterHelper.CreateToDateFor24HoursWithAnd(
                    endOrderDate, parameters, "To2", "OrderDate" );
            }
            else
            {
                from2 = DateFilterHelper.CreateFromDateWithAnd(
                    startOrderDate, parameters, "From2", "OrderDate" );
                to2 = DateFilterHelper.CreateToDateWithAnd(
                    endOrderDate, parameters, "To2", "OrderDate" );
            }

            string storeFilter2 = "";
            if (storeID != "0")
            {
                storeFilter2 = "AND StoreID = @StoreID2 ";
                parameters.Add( DataAccess.CreateParameterString( storeID ) );
            }

            if (!String.IsNullOrEmpty( sortBy ))
            {
                sort = CreateSortString( sortBy );
            }
            else
            {
                sort = "";
            }

            string script = "SELECT O.OrderDate, O.Period, Total, NumberOfOrder, Average, Quantity, " +
                "TotalTax, TotalShippingCost, " +
                DbImplementation.Sql.CreateDivideFucntion( "Quantity", "NumberOfOrder" ) + " as AvgQuantity " +
                "FROM( SELECT " + DbImplementation.Sql.CreateDateFormat( "OrderDate" ) + " as OrderDate, " +
                DbImplementation.Sql.CreateHourFormat( "OrderDate" ) + " as Period, " +
                "SUM" + CreateTotalSqlNoAs() + " AS Total, " +
                "Count(Orders.OrderID) as NumberOfOrder, " +
                "AVG" + CreateTotalSqlNoAs() + " AS Average, " +
                "Sum (Tax) as TotalTax, Sum (ShippingCost) as TotalShippingCost " +
                "FROM Orders " +
                "WHERE PaymentComplete = @PaymentComplete " +
                "AND Processed = @Processed And Cancelled = @Cancelled " +
                from1 + to1 + storeFilter1 +
                "Group By " +
                DbImplementation.Sql.CreateDateFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateHourFormat( "OrderDate" ) +
                " ) O " +
                "INNER JOIN (SELECT " + DbImplementation.Sql.CreateDateFormat( "OrderDate" ) + " as OrderDate, " +
                DbImplementation.Sql.CreateHourFormat( "OrderDate" ) + " as Period, " +
                "SUM(Quantity) as Quantity " +
                "FROM Orders, OrderItem " +
                "WHERE PaymentComplete = @PaymentComplete1 " +
                "AND Processed = @Processed1 And Cancelled = @Cancelled1 " +
                from2 + to2 + storeFilter2 +
                "AND Orders.OrderID = OrderItem.OrderID " +
                "Group By " +
                DbImplementation.Sql.CreateDateFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateHourFormat( "OrderDate" ) +
                ") OI " +
                "On O.Period = OI.Period and O.OrderDate = OI.OrderDate " +
                sort;
            return DataAccess.ExecuteSelect( script,
                DataAccess.ConvertArrayListToDbParameters( parameters ) );
        }

        private DataTable Get24HoursSaleReportTable(
            string sortBy,
            DateTime startOrderDate,
            DateTime endOrderDate,
            string storeID )
        {
            DataTable saleTable = GetOneDayTable( sortBy, startOrderDate, endOrderDate, "24Hours", storeID );
            return saleTable;
        }

        private DataTable GetOneDaySaleReportTable(
           string sortBy,
           DateTime? startOrderDate,
           DateTime? endOrderDate,
           string storeID )
        {
            DataTable saleTable = GetOneDayTable( sortBy, startOrderDate, endOrderDate, "OneDay", storeID );
            return saleTable;
        }

        private DataTable GetYearSaleReportTable(
           string sortBy,
           DateTime? startOrderDate,
           DateTime? endOrderDate,
           string storeID )
        {
            ArrayList parameters = new ArrayList();

            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );
            string from1 = DateFilterHelper.CreateFromDateWithAnd(
                startOrderDate, parameters, "From1", "OrderDate" );
            string to1 = DateFilterHelper.CreateToDateWithAnd(
                endOrderDate, parameters, "To1", "OrderDate" );
            string storeFilter1 = "";
            if (storeID != "0")
            {
                storeFilter1 = "AND StoreID = @StoreID1 ";
                parameters.Add( DataAccess.CreateParameterString( storeID ) );
            }
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );
            string from2 = DateFilterHelper.CreateFromDateWithAnd(
                startOrderDate, parameters, "From2", "OrderDate" );
            string to2 = DateFilterHelper.CreateToDateWithAnd(
                endOrderDate, parameters, "To2", "OrderDate" );
            string storeFilter2 = "";
            if (storeID != "0")
            {
                storeFilter2 = "AND StoreID = @StoreID2 ";
                parameters.Add( DataAccess.CreateParameterString( storeID ) );
            }

            string sort;

            if (!String.IsNullOrEmpty( sortBy ))
            {
                if (sortBy.Contains( "Period" ))
                {
                    string[] sortWord = sortBy.Split( ' ' );
                    sortBy = "SortDate " + sortWord[1];
                }
                sort = CreateSortString( sortBy );
            }
            else
            {
                sort = "";
            }

            string script =
                "SELECT O.[MonthNumber], O.[MonthName], O.[Year], O.Period, Total, NumberOfOrder, Average, " +
                "Quantity, O.SortDate, TotalTax, TotalShippingCost, " +
                DbImplementation.Sql.CreateDivideFucntion( "Quantity", "NumberOfOrder" ) + " as AvgQuantity " +
                "FROM ( SELECT " +
                DbImplementation.Sql.CreateMonthNumberFormat( "OrderDate" ) + " as [MonthNumber], " +
                DbImplementation.Sql.CreateMonthNameFormat( "OrderDate" ) + " as [MonthName], " +
                DbImplementation.Sql.CreateYearFormat( "OrderDate" ) + " as [Year], " +
                DbImplementation.Sql.CreateMonthYearFormat( "OrderDate" ) + " as Period, " +
                DbImplementation.Sql.CreateYearMonthFormat( "OrderDate" ) + " as SortDate, " +
                " SUM" + CreateTotalSqlNoAs() + " AS Total, " +
                " Count(Orders.OrderID) as NumberOfOrder, " +
                " AVG" + CreateTotalSqlNoAs() + " AS Average, " +
                " Sum (Tax) as TotalTax, Sum (ShippingCost) as TotalShippingCost " +
                " FROM Orders " +
                "WHERE PaymentComplete = @PaymentComplete " +
                "AND Processed = @Processed And Cancelled = @Cancelled " +
                from1 +
                to1 +
                storeFilter1 +
                " GROUP BY " +
                DbImplementation.Sql.CreateMonthNumberFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateMonthNameFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateYearFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateMonthYearFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateYearMonthFormat( "OrderDate" ) +
                " ) O INNER JOIN " +
                "  (SELECT " +
                DbImplementation.Sql.CreateMonthNumberFormat( "OrderDate" ) + " as [MonthNumber], " +
                DbImplementation.Sql.CreateMonthNameFormat( "OrderDate" ) + " as [MonthName], " +
                DbImplementation.Sql.CreateYearFormat( "OrderDate" ) + " as [Year], " +
                DbImplementation.Sql.CreateMonthYearFormat( "OrderDate" ) + " as Period, " +
                DbImplementation.Sql.CreateYearMonthFormat( "OrderDate" ) + " as SortDate, " +
                " SUM(Quantity) as Quantity " +
                " FROM Orders, OrderItem " +
                " WHERE PaymentComplete = @PaymentComplete1 " +
                " AND Processed = @Processed1 And Cancelled = @Cancelled1 " +
                from2 +
                to2 +
                storeFilter2 +
                "AND Orders.OrderID = OrderItem.OrderID " +
                " GROUP BY " +
                DbImplementation.Sql.CreateMonthNumberFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateMonthNameFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateYearFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateMonthYearFormat( "OrderDate" ) + " , " +
                DbImplementation.Sql.CreateYearMonthFormat( "OrderDate" ) + " ) OI " +
                " On O.[MonthNumber] = OI.[MonthNumber] and O.[MonthName] = OI.[MonthName] and " +
                " O.[Year] = OI.[Year] and O.Period = OI.Period and O.SortDate = OI.SortDate " +
                sort;

            return DataAccess.ExecuteSelect( script,
                 DataAccess.ConvertArrayListToDbParameters( parameters ) );
        }

        private DataTable GetSaleReportTable(
          string sortBy,
          DateTime? startOrderDate,
          DateTime? endOrderDate,
          string storeID )
        {
            ArrayList parameters = new ArrayList();

            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );
            string from1 = DateFilterHelper.CreateFromDateWithAnd(
                startOrderDate, parameters, "From1", "OrderDate" );
            string to1 = DateFilterHelper.CreateToDateWithAnd(
                endOrderDate, parameters, "To1", "OrderDate" );

            string storeFilter1 = "";
            if (storeID != "0")
            {
                storeFilter1 = "AND StoreID = @StoreID1 ";
                parameters.Add( DataAccess.CreateParameterString( storeID ) );
            }

            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );
            string from2 = DateFilterHelper.CreateFromDateWithAnd(
                startOrderDate, parameters, "From2", "OrderDate" );
            string to2 = DateFilterHelper.CreateToDateWithAnd(
                endOrderDate, parameters, "To2", "OrderDate" );

            string storeFilter2 = "";
            if (storeID != "0")
            {
                storeFilter2 = "AND StoreID = @StoreID2 ";
                parameters.Add( DataAccess.CreateParameterString( storeID ) );
            }

            string sort;

            if (!String.IsNullOrEmpty( sortBy ))
            {
                sort = CreateSortString( sortBy );
            }
            else
            {
                sort = "";
            }

            string script =
                "SELECT O.Period, Total, NumberOfOrder, Average, Quantity, " + "TotalTax, TotalShippingCost, " +
                DbImplementation.Sql.CreateDivideFucntion( "Quantity", "NumberOfOrder" ) + " as AvgQuantity " +
                "FROM ( SELECT " +
                DbImplementation.Sql.CreateDateFormat( "OrderDate" ) + " as Period, " +
                "       SUM" + CreateTotalSqlNoAs() + " AS Total, COUNT(Orders.OrderID) as NumberOfOrder, " +
                "       AVG" + CreateTotalSqlNoAs() + " AS Average, " +
                "       Sum (Tax) as TotalTax, Sum (ShippingCost) as TotalShippingCost " +
                "       FROM Orders " +
                "       WHERE PaymentComplete = @PaymentComplete " +
                "       AND Processed = @Processed And Cancelled = @Cancelled " +
                        from1 +
                        to1 +
                        storeFilter1 +
                "       GROUP BY " + DbImplementation.Sql.CreateDateFormat( "OrderDate" ) + " ) O INNER JOIN ( " +
                "           SELECT " +
                DbImplementation.Sql.CreateDateFormat( "OrderDate" ) + " as Period, " +
                "           SUM(Quantity) as Quantity " +
                "           FROM Orders, OrderItem " +
                "           WHERE PaymentComplete = @PaymentComplete1 " +
                "           AND Processed = @Processed1 And Cancelled = @Cancelled1 " +
                            from2 +
                            to2 +
                            storeFilter2 +
                "           and Orders.OrderID = OrderItem.OrderID " +
                "           GROUP BY " + DbImplementation.Sql.CreateDateFormat( "OrderDate" ) +
                "           ) OI ON O.Period = OI.Period  " +
                sort;
            return DataAccess.ExecuteSelect( script,
                DataAccess.ConvertArrayListToDbParameters( parameters ) );
        }

        public DataTable GetSaleReport( DateTime? startOrderDate, DateTime? endOrderDate, string storeID )
        {
            return GetSaleReportTable( "", startOrderDate, endOrderDate, storeID );
        }

        private DataTable GetSaleReport(
            string sortBy,
            DateTime? startOrderDate,
            DateTime? endOrderDate,
            int startIndex,
            int endIndex,
            string storeID,
            out int howManyItems )
        {

            DataTable full = GetSaleReportTable( sortBy, startOrderDate, endOrderDate, storeID );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["Period"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable GetOneDaySaleReport( DateTime? startOrderDate, DateTime? endOrderDate, string storeID )
        {
            return GetOneDaySaleReportTable( "", startOrderDate, endOrderDate, storeID );
        }

        private DataTable GetOneDaySaleReport(
            string sortBy,
            DateTime? startOrderDate,
            DateTime? endOrderDate,
            int startIndex,
            int endIndex,
            string storeID,
            out int howManyItems )
        {

            DataTable full = GetOneDaySaleReportTable( sortBy, startOrderDate, endOrderDate, storeID );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["Period"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable Get24HoursSaleReport( DateTime startOrderDate, DateTime endOrderDate, string storeID )
        {
            return Get24HoursSaleReportTable( "", startOrderDate, endOrderDate, storeID );
        }

        private DataTable Get24HoursSaleReport(
            string sortBy,
            DateTime startOrderDate,
            DateTime endOrderDate,
            int startIndex,
            int endIndex,
            string storeID,
            out int howManyItems )
        {

            DataTable full = Get24HoursSaleReportTable( sortBy, startOrderDate, endOrderDate, storeID );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["Period"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable GetYearSaleReport( DateTime? startOrderDate, DateTime? endOrderDate, string storeID )
        {
            return GetYearSaleReportTable( "", startOrderDate, endOrderDate, storeID );
        }

        private DataTable GetYearSaleReport(
              string sortBy,
            DateTime? startOrderDate,
            DateTime? endOrderDate,
            int startIndex,
            int endIndex,
            string storeID,
            out int howManyItems )
        {

            DataTable full = GetYearSaleReportTable( sortBy, startOrderDate, endOrderDate, storeID );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["Period"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable GetReportData(
            string sortBy,
            PeriodType period,
            DateTime startOrderDate,
            DateTime endOrderDate,
            int startIndex,
            int endIndex,
            string storeID,
            out int totalItems )
        {
            DataTable table;
            if (period == PeriodType.Custom)
            {
                period = ReportFilterUtilities.ResolveCustomPeriodType( startOrderDate, endOrderDate );
            }

            if (period == PeriodType.Yesterday || period == PeriodType.Today)
            {
                table = GetOneDaySaleReport( sortBy, startOrderDate, endOrderDate,
                            startIndex, endIndex, storeID, out totalItems );
            }
            else if (period == PeriodType.Last24Hours)
            {
                table = Get24HoursSaleReport( sortBy, startOrderDate, endOrderDate,
                            startIndex, endIndex, storeID, out totalItems );
            }
            else if (period == PeriodType.ThisYear || period == PeriodType.Last3Years)
            {
                table = GetYearSaleReport( sortBy, startOrderDate, endOrderDate,
                            startIndex, endIndex, storeID, out totalItems );
            }
            else
            {
                table = GetSaleReport( sortBy, startOrderDate, endOrderDate,
                            startIndex, endIndex, storeID, out totalItems );
            }
            return table;
        }

        public DataTable GetReportData( PeriodType period, DateTime startOrderDate, DateTime endOrderDate, string storeID )
        {
            DataTable table;
            if (period == PeriodType.Custom)
            {
                period = ReportFilterUtilities.ResolveCustomPeriodType( startOrderDate, endOrderDate );
            }

            if (period == PeriodType.Yesterday || period == PeriodType.Today)
            {
                table = GetOneDaySaleReport( startOrderDate, endOrderDate, storeID );
            }
            else if (period == PeriodType.Last24Hours)
            {
                table = Get24HoursSaleReport( startOrderDate, endOrderDate, storeID );
            }
            else if (period == PeriodType.ThisYear || period == PeriodType.Last3Years)
            {
                table = GetYearSaleReport( startOrderDate, endOrderDate, storeID );
            }
            else
            {
                table = GetSaleReport( startOrderDate, endOrderDate, storeID );
            }
            return table;
        }
    }
}
