﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using Vevo.WebServices.Base;
using Vevo.WebServices.Security;
using Vevo.WebServices.SearchFilters;
using Vevo.WebAppLib;
using Vevo.Domain.Payments;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain;
using System.Data;
using Vevo.Base.Domain;

namespace Vevo.WebServices.GiftCertificates
{
    public class GiftCertificateService : IGiftCertificateService
    {
        private GiftCertificateConstraints ConvertGiftCertificateToConstraints( GiftCertificate giftCertificate )
        {
            GiftCertificateConstraints constraints = new GiftCertificateConstraints();
            constraints.ExpireDate = giftCertificate.ExpireDate;
            constraints.GiftCertificateCode = giftCertificate.GiftCertificateCode;
            constraints.GiftValue = giftCertificate.GiftValue;
            constraints.IsActive = giftCertificate.IsActive;
            constraints.IsExpirable = giftCertificate.IsExpirable;
            constraints.NeedPhysical = giftCertificate.NeedPhysical;
            constraints.OrderItemID = giftCertificate.OrderItemID;
            constraints.PersonalNote = giftCertificate.PersonalNote;
            constraints.ProductID = giftCertificate.ProductID;
            constraints.Recipient = giftCertificate.Recipient;
            constraints.RemainValue = giftCertificate.RemainValue;
            constraints.UrlParameter = giftCertificate.UrlParameter;

            return constraints;
        }

        public GiftCertificateListResponseMessage ListGiftCertificates( GiftCertificateListRequestMessage ListMessage )
        {
            GiftCertificateListResponseMessage responseMessage = new GiftCertificateListResponseMessage();
            if ( WcfSessionPool.GetSession( ListMessage.Ticket ) == null )
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }
            try
            {
                SearchFilter searchFilter = SearchFilterUtilities.ConvertConstraintToSearchFilter(
                                ListMessage.SearchFilter );
                int giftCertificateCount = 0;
                DataTable table = DataAccessContext.GiftCertificateRepository.SearchGiftCertificate(
                    "GiftCertificateCode", searchFilter, ListMessage.SearchFilter.StartIndex, ListMessage.SearchFilter.EndIndex, out giftCertificateCount );

                GiftCertificate giftCertificate;
                responseMessage.GiftCertificates = new List<GiftCertificateConstraints>();
                foreach ( DataRow row in table.Rows )
                {
                    giftCertificate = new GiftCertificate();
                    giftCertificate.ExpireDate = Convert.ToDateTime( row[ "ExpireDate" ] );
                    giftCertificate.GiftCertificateCode = row[ "GiftCertificateCode" ].ToString();
                    giftCertificate.GiftValue = Convert.ToDecimal( row[ "GiftValue" ] );
                    giftCertificate.IsActive = Convert.ToBoolean( row[ "IsActive" ] );
                    giftCertificate.IsExpirable = Convert.ToBoolean( row[ "IsExpirable" ] );
                    giftCertificate.NeedPhysical = Convert.ToBoolean( row[ "NeedPhysical" ] );
                    giftCertificate.OrderItemID = row[ "OrderItemID" ].ToString();
                    giftCertificate.PersonalNote = row[ "PersonalNote" ].ToString();
                    giftCertificate.ProductID = row[ "ProductID" ].ToString();
                    giftCertificate.Recipient = row[ "Recipient" ].ToString();
                    giftCertificate.RemainValue = Convert.ToDecimal( row[ "RemainValue" ] );
                    giftCertificate.UrlParameter = row[ "UrlParameter" ].ToString();
                    responseMessage.GiftCertificates.Add( ConvertGiftCertificateToConstraints( giftCertificate ) );
                }
            }
            catch ( Exception e )
            {
                WebUtilities.LogError( e );
                responseMessage.ErrorMessage = "Error: Cannot get stores from repository";
            }

            return responseMessage;
        }
    }
}
