using System;
using System.Data;
using System.Configuration;
using System.Collections.Specialized;
using System.Globalization;
using System.Threading;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo.Domain;
using Vevo.WebAppLib;
using Vevo.WebUI;

namespace Vevo
{
    /// <summary>
    /// Summary description for CurrencyUtilities
    /// </summary>
    public static class CurrencyUtilities
    {
        #region Private
        private const string CurrencyName = "Dollars";
        private static string _defaultCode;

        private static HttpSessionState Session
        {
            get
            {
                return HttpContext.Current.Session;
            }
        }

        private static bool IsDisplayCodeExist( string currencyCode )
        {
            Currency currency = DataAccessContext.CurrencyRepository.GetOne( currencyCode );

            if (currency == Currency.Null)
                return false;
            else
                return true;
        }

        private static string DefaultCurrencyCode
        {
            get
            {
                if (_defaultCode == null)
                {
                    if (SystemConfig.ContainsKey( "BaseWebsiteCurrency" ))
                        _defaultCode = DataAccessContext.Configurations.GetValue( "BaseWebsiteCurrency" );

                    else
                        _defaultCode = DataAccessContext.CurrencyRepository.GetCodeByName( CurrencyName );
                }

                return _defaultCode;
            }
        }

        private static void RefreshBaseCurrency()
        {
            HttpRuntime.Cache.Remove( "BaseCurrencyCode" );
            HttpRuntime.Cache.Remove( "BaseCurrencySymbol" );
        }
        #endregion


        public static string StoreCurrencyCode
        {
            get
            {
                return StoreContext.StoreCurrencyCode;
            }
            set
            {
                StoreContext.StoreCurrencyCode = value;
                ClearCache();
            }
        }

        public static string StoreCurrencySymbol
        {
            get
            {
                Currency currency = DataAccessContext.CurrencyRepository.GetOne(
                    CurrencyUtilities.StoreCurrencyCode );
                return currency.CurrencySymbol;
            }
        }

        public static double StoreConversionRate
        {
            get
            {
                Currency currency = DataAccessContext.CurrencyRepository.GetOne(
                    CurrencyUtilities.StoreCurrencyCode );
                return currency.ConversionRate;
            }
        }

        public static string BaseCurrencyCode
        {
            get
            {
                object baseCurrency = HttpRuntime.Cache["BaseCurrencyCode"];
                if (baseCurrency == null)
                {
                    baseCurrency = DefaultCurrencyCode;
                    HttpRuntime.Cache["BaseCurrencyCode"] = baseCurrency;
                }

                return baseCurrency.ToString();
            }
            set
            {
                ClearCache();
                HttpRuntime.Cache["BaseCurrencyCode"] = value;
            }
        }

        public static string BaseCurrencySymbol
        {
            get
            {
                Currency currency = DataAccessContext.CurrencyRepository.GetOne(
                    CurrencyUtilities.BaseCurrencyCode );
                return currency.CurrencySymbol;
            }
        }
        public static string BaseCurrencyPosition
        {
            get
            {
                Currency currency = DataAccessContext.CurrencyRepository.GetOne(
                    CurrencyUtilities.BaseCurrencyCode );

                return currency.CurrencyPosition;
            }
        }
        public static void ClearCache()
        {
            RefreshBaseCurrency();
        }

        public static bool IsWarningPaymentCurrency( string paymentGatewayCurrency )
        {
            if (String.IsNullOrEmpty( paymentGatewayCurrency ))
                return false;

            Currency currency = DataAccessContext.CurrencyRepository.GetOne( paymentGatewayCurrency );
            if (currency.ConversionRate != 1.0)
                return true;
            else
                return false;
        }

    }
}
