using System;
using System.Collections.Generic;
using System.Web;
using System.Text;
using System.IO;
using Vevo.DataAccessLib;
using Vevo.Shared.DataAccess;
using Vevo.Shared.DataAccess.Tester;
using Vevo.Domain;

namespace Vevo.WebAppLib
{
    public class ExceptionLog
    {
        protected const int HttpResourceNotFoundErrorCode = 404;

        protected Exception _ex;
        protected StringBuilder _sb;

        private void SaveErrorLog()
        {
            _sb = new StringBuilder();

			try
            {
            WriteExceptionHeader();
			}
			catch
			{
			}

            Exception currentEx = _ex;
            while (currentEx != null)
            {
                if (IsInformativeException( currentEx ) ||
                    IsLastException( currentEx ))
                {
                    WriteExceptionDetails( currentEx );
                }

                currentEx = currentEx.InnerException;
            }

            string siteName = "";

            try
            {
                siteName = DataAccessContext.Configurations.GetValue( "SiteName" );
            }
            catch
            {

            }

            string subject = siteName + " error report";
            string body = _sb.ToString();


            WriteLogFile( body );
        }

        protected void WriteExceptionHeader()
        {
            string dateTime = DateTime.Now.ToLongDateString() + ", at "
                            + DateTime.Now.ToShortTimeString();

            _sb.Append( "Exception generated on " );
            _sb.Append( dateTime );
            _sb.AppendLine();
            _sb.Append( "Page location: " );
            _sb.Append( HttpContext.Current.Request.RawUrl );
        }

        protected bool IsHttpPageNotFound( Exception ex )
        {
            return (ex is HttpException &&
                ((HttpException) ex).GetHttpCode() == HttpResourceNotFoundErrorCode);
        }

        protected void WriteExceptionDetails( Exception ex )
        {
            _sb.AppendLine();
            _sb.Append( "------------------------------" );

            if (ex is HttpException)
            {
                _sb.AppendLine();
                _sb.Append( "Http Code: " );
                _sb.Append( ((HttpException) ex).GetHttpCode() );
            }

            _sb.AppendLine();
            _sb.Append( "Message: " );
            _sb.Append( ex.Message );
            _sb.AppendLine();
            _sb.Append( "Visitor IP address: " );
            _sb.Append( WebUtilities.GetVisitorIP() );
            _sb.AppendLine();
            _sb.Append( "Exception Type: " );
            _sb.Append( ex.GetType().ToString() );
            _sb.AppendLine();
            _sb.Append( "nMethod: " );
            _sb.Append( ex.TargetSite );

            if (!IsHttpPageNotFound( ex ))
            {
                _sb.AppendLine();
                _sb.Append( "Stack Trace: " );
                _sb.AppendLine();
                _sb.Append( ex.StackTrace );
            }
        }

        protected bool IsInformativeException( Exception ex )
        {
            return ex.GetType() != typeof( HttpUnhandledException );
        }

        protected bool IsLastException( Exception ex )
        {
            return ex.InnerException == null;
        }

        public ExceptionLog( Exception ex )
        {
            _ex = ex;
        }

        public void Save()
        {
            SaveErrorLog();
        }

        private static void WriteLogFile( string text )
        {
            StreamWriter fp;
            try
            {
                fp = new StreamWriter( HttpContext.Current.Server.MapPath( "~/App_Data/" ) + "log.txt", true );
                fp.WriteLine( "-------------------------" );
                fp.WriteLine( text );
                fp.WriteLine( "" );
                fp.Close();

            }
            catch (Exception)
            {
            }
        }
    }
}
